@@ -26,10 +26,13 @@ from timedinput import timedinput
 from grpc import RpcError
 from grpc import StatusCode
 
+from leaf_common.config.file_of_class import FileOfClass
+
 from neuro_san.client.agent_session_factory import AgentSessionFactory
+from neuro_san.client.concierge_session_factory import ConciergeSessionFactory
 from neuro_san.client.streaming_input_processor import StreamingInputProcessor
 from neuro_san.interfaces.agent_session import AgentSession
-from neuro_san.internals.utils.file_of_class import FileOfClass
+from neuro_san.interfaces.concierge_session import ConciergeSession
 
 
 class AgentCli:
@@ -58,6 +61,12 @@ class AgentCli:
         Main entry point for command line user interaction
         """
         self.parse_args()
+
+        # See if we are doing a list operation
+        if self.args.list or self.args.tags or self.args.tag:
+            self.list()
+            return
+
         self.open_session()
         if self.args.connectivity:
             self.connectivity()
@@ -75,6 +84,13 @@ class AgentCli:
         for connectivity_info in response.get("connectivity_info", empty_list):
             print(f"{json.dumps(connectivity_info, indent=4, sort_keys=True)}")
 
+        metadata = response.get("metadata")
+        metadata_str: str = "<None>"
+        if metadata is not None:
+            metadata_str = json.dumps(metadata, indent=4, sort_keys=True)
+
+        print(f"\nNetwork Metadata: {metadata_str}")
+
     def chat(self):
         """
         Perform chat function
@@ -197,6 +213,10 @@ Some suggestions:
                 pretty_sly: str = json.dumps(state.get('sly_data'), indent=4, sort_keys=True)
                 print(f"Returned sly_data is: {pretty_sly}")
 
+            if self.args.tokens:
+                print("Token Accounting:")
+                print(f"{json.dumps(state.get('token_accounting'), indent=4, sort_keys=True)}")
+
             if self.args.response_output_file is not None:
                 output_path: Path = Path(self.args.response_output_file)
                 with output_path.open('w', encoding="utf-8") as output_file:
@@ -234,6 +254,16 @@ Some suggestions:
         arg_parser.add_argument("--agent", type=str, default="esp_decision_assistant",
                                 help="Name of the agent to talk to")
 
+        # What agents can we connect to?
+        group = arg_parser.add_argument_group(title="Agent Listings",
+                                              description="What agents can we connect to?")
+        group.add_argument("--list", default=False, dest="list", action="store_true",
+                           help="List all available agents")
+        group.add_argument("--tag", type=str,
+                           help="List all agents marked with the given tag.")
+        group.add_argument("--tags", default=False, dest="tags", action="store_true",
+                           help="List all tags associated with agents")
+
         # How will we connect to neuro-san?
         group = arg_parser.add_argument_group(title="Session Type",
                                               description="How will we connect to neuro-san?")
@@ -333,6 +363,8 @@ Have external tools that can be found in the local agent manifest use a service
         group = arg_parser.add_argument_group(title="Other Modes of Operation")
         group.add_argument("--connectivity", default=False, dest="connectivity", action="store_true",
                            help="List the connectivity of agent network nodes")
+        group.add_argument("--tokens", default=False, dest="tokens", action="store_true",
+                           help="Print token accounting info with every response")
         self.arg_groups[group.title] = group
 
     def open_session(self):
@@ -400,6 +432,46 @@ Have external tools that can be found in the local agent manifest use a service
 
         return False
 
+    def list(self):
+        """
+        List available agents via a concierge session
+        """
+        hostname = "localhost"
+        if self.args.host is not None or not self.args.local:
+            hostname = self.args.host
+
+        metadata: Dict[str, str] = {
+            "user_id": self.args.user_id
+        }
+
+        concierge_session: ConciergeSession = ConciergeSessionFactory().create_session(
+            self.args.connection,
+            hostname=hostname,
+            port=self.args.port,
+            metadata=metadata,
+            connect_timeout_in_seconds=self.args.timeout)
+
+        request_dict: Dict[str, Any] = {}
+        response_dict: Dict[str, Any] = concierge_session.list(request_dict)
+
+        empty_list: List[Dict[str, Any]] = []
+        if self.args.tags:
+            tags = set()
+            for agent_info in response_dict.get("agents", empty_list):
+                agent_tags: List[str] = agent_info.get("tags", empty_list)
+                tags.update(agent_tags)
+            print(f"Available tags:\n{json.dumps(list(tags), indent=4, sort_keys=True)}")
+
+        elif self.args.tag:
+            print(f"Available agents for tag {self.args.tag}:\n")
+            for agent_info in response_dict.get("agents", empty_list):
+                agent_tags: List[str] = agent_info.get("tags", empty_list)
+                if self.args.tag in agent_tags:
+                    print(json.dumps(agent_info, indent=4, sort_keys=True))
+
+        else:
+            print(f"Available agents:\n{json.dumps(response_dict, indent=4, sort_keys=True)}")
+
 
 if __name__ == '__main__':
     AgentCli().main()
